// The following program calculates the number of coefficients
// needed to define a multivariate polynomial

#include <iostream>

#include "CmdLine.hh"
#include "npstat/nm/binomialCoefficient.hh"

using namespace std;

static void print_usage(const char* progname)
{
    cout << "\nUsage: " << progname << " dim deg\n" << endl;
    cout << "The following arguments are required:" << endl;
    cout << "  dim   is the dimensionality of the space" << endl;
    cout << "  deg   is the maximum polynomial degree" << endl;
    cout << endl;
}

int main(int argc, char** argv)
{
    CmdLine cmdline(argc, argv);

    // If the program is executed without any arguments,
    // print its usage instructions and exit
    if (argc == 1)
    {
        print_usage(cmdline.progname());
        return 0;
    }

    // Parse input arguments
    unsigned mydim, upDeg;
    try {
        cmdline.optend();
        if (cmdline.argc() != 2)
            throw CmdLineError("wrong number of command line arguments");
        cmdline >> mydim >> upDeg;
    }
    catch (const CmdLineError& e) {
        cerr << "Error in " << cmdline.progname() << ": "
             << e.str() << endl;
        print_usage(cmdline.progname());
        return 1;
    }
    if (!mydim)
    {
        cerr << "Error in " << cmdline.progname()
             << ": dimensionality of the space must be positive" << endl;
        return 1;
    }

    std::cout << npstat::binomialCoefficient(mydim+upDeg, upDeg) << std::endl;
    return 0;
}
