#!/usr/bin/env python3
"""
This example illustrates Johnson system of frequency curves. These
curves can attain all possible values of skewness and kurtosis. See
"plot_density_1d_matplotlib.py" script for additional information
on plotting 1-d densities.
"""

__author__="Igor Volobouev (i.volobouev@ttu.edu)"
__version__="1.0"
__date__ ="Sep 21 2019"

# Perform necessary imports
import npstat as ns
import matplotlib.pyplot as plt

# Some script parameters
xmin = -4.0
xmax =  6.0
npoints = 1001

# For all densities plotted below, the mean is 0 and
# the standard deviation is 1
skewness = ( 0,   0, -0.5, 0.3,  3 )
kurtosis = ( 3,   10,  2,  1.7,  30)
color    = ('k', 'g', 'r', 'm', 'b')
for s, k, c in zip(skewness, kurtosis, color):
    d = ns.JohnsonSystem(0, 1, s, k)
    x, y = ns.scanDensity1D(d, xmin, xmax, npoints)
    plt.plot(x, y, c, label="s = {}, k = {}".format(s, k))

# Set the axis labels and the plot title
plt.xlabel('X')
plt.ylabel('Probability density')
plt.title('Johnson Curves')
plt.legend()

# Display the plot window
plt.show()
