#include <stdexcept>

#include "npstat/nm/Gauss1DQuadrature.hh"

namespace npstat {
    Gauss1DQuadrature::Gauss1DQuadrature(const unsigned npoints,
                                         const long double i_mu,
                                         const long double i_sigma)
        : ghq_(npoints), mean_(i_mu), sigma_(i_sigma)
    {
        if (sigma_ <= 0.0) throw std::invalid_argument(
            "In npstat::Gauss1DQuadrature constructor: "
            "sigma parameter must be positive");
    }

    void Gauss1DQuadrature::getAllAbscissae(long double* abscissae,
                                            const unsigned len) const
    {
        const long double sqr2 = 1.41421356237309504880168872421L;
        ghq_.getAllAbscissae(abscissae, len);
        for (unsigned i=0; i<len; ++i)
            abscissae[i] = mean_ + sqr2*sigma_*abscissae[i];
    }

    void Gauss1DQuadrature::getAllWeights(long double* weights,
                                          const unsigned len) const
    {
        ghq_.getAllWeights(weights, len);
        for (unsigned i=0; i<len; ++i)
            weights[i] /= 1.77245385090551602729816748334L;
    }
}
