#ifndef NPSTAT_PRECISETYPE_HH_
#define NPSTAT_PRECISETYPE_HH_

/*!
// \file PreciseType.hh
//
// \brief Compile-time deduction of an appropriate precise numeric type
//
// Author: I. Volobouev
//
// January 2012
*/

#include <complex>

#include "geners/IOIsNumber.hh"
#include "npstat/nm/lapack_double.h"

namespace npstat {
    template <class T, int I=0>
    struct PreciseTypeHelper
    {
        typedef T type;
    };

#ifdef USE_LAPACK_QUAD
    template <class T>
    struct PreciseTypeHelper<T, 1>
    {
        typedef lapack_double type;
    };
#else
    template <class T>
    struct PreciseTypeHelper<T, 1>
    {
        typedef long double type;
    };
#endif

    /**
    // Use "long double" as the most precise type among various built-in
    // numeric types, std::complex<long double> for complex types, and
    // the type itself for other types.
    */
    template <class T>
    struct PreciseType
    {
        typedef typename PreciseTypeHelper<T,gs::IOIsNumber<T>::value>::type type;
    };

#ifdef USE_LAPACK_QUAD
    template <class T>
    struct PreciseType<std::complex<T> >
    {
        typedef std::complex<lapack_double> type;
    };

    template <class T>
    struct PreciseType<const std::complex<T> >
    {
        typedef std::complex<lapack_double> type;
    };

    template <class T>
    struct PreciseType<volatile std::complex<T> >
    {
        typedef std::complex<lapack_double> type;
    };

    template <class T>
    struct PreciseType<const volatile std::complex<T> >
    {
        typedef std::complex<lapack_double> type;
    };
#else
    template <class T>
    struct PreciseType<std::complex<T> >
    {
        typedef std::complex<long double> type;
    };

    template <class T>
    struct PreciseType<const std::complex<T> >
    {
        typedef std::complex<long double> type;
    };

    template <class T>
    struct PreciseType<volatile std::complex<T> >
    {
        typedef std::complex<long double> type;
    };

    template <class T>
    struct PreciseType<const volatile std::complex<T> >
    {
        typedef std::complex<long double> type;
    };
#endif
}

#endif // NPSTAT_PRECISETYPE_HH_
