#include <cassert>
#include <stdexcept>

#include "npstat/nm/RectangleQuadrature1D.hh"

namespace npstat {
    RectangleQuadrature1D::RectangleQuadrature1D(const unsigned npt)
        : npoints_(npt)
    {
        if (!npoints_) throw std::invalid_argument(
            "In npstat::RectangleQuadrature1D constructor: "
            "number of integration points must be positive");
    }

    void RectangleQuadrature1D::getAllAbscissae(
        long double* abscissae, unsigned len) const
    {
        if (len < npoints_) throw std::invalid_argument(
            "In npstat::RectangleQuadrature1D::getAllAbscissae: "
            "unsifficient length of the output buffer");
        assert(abscissae);
        const long double step = 2.0L/npoints_;
        for (unsigned i=0; i<npoints_; ++i)
            abscissae[i] = -1.0L + step*(i + 0.5L);
    }

    void RectangleQuadrature1D::getAllWeights(
        long double* weights, unsigned len) const
    {
        if (len < npoints_) throw std::invalid_argument(
            "In npstat::RectangleQuadrature1D::getAllWeights: "
            "unsifficient length of the output buffer");
        assert(weights);
        const long double w = 2.0L/npoints_;
        for (unsigned i=0; i<npoints_; ++i)
            weights[i] = w;
    }
}
