#include <cassert>
#include <vector>

#include "npstat/nm/performSchultzIteration.hh"

namespace npstat {
    namespace Private {
        void squareMatMul(const long double* a,
                          const long double* b,
                          const unsigned dim,
                          long double* c)
        {
            assert(a);
            assert(b);
            assert(dim);
            assert(c);
            assert(c != a && c != b);

            for (unsigned row=0; row<dim; ++row)
            {
                const long double* prow = a + row*dim;
                long double* pres = c + row*dim;
                for (unsigned col=0; col<dim; ++col)
                {
                    long double sum = 0.0L;
                    const long double* slice = b + col;
                    for (unsigned k=0; k<dim; ++k)
                        sum += prow[k]*slice[k*dim];
                    pres[col] = sum;
                }
            }
        }
    }

    void performSchultzIteration(const long double* mat,
                                 const unsigned dim,
                                 long double* inverse)
    {
        // The following code is not thread safe!
        static std::vector<long double> ldbufv;
        
        const unsigned len = dim*dim;
        if (ldbufv.size() < 2U*len)
            ldbufv.resize(2U*len);
        long double* buf1 = &ldbufv[0];
        long double* buf2 = buf1 + len;
        Private::squareMatMul(inverse, mat, dim, buf1);
        Private::squareMatMul(buf1, inverse, dim, buf2);
        for (unsigned i=0; i<len; ++i)
            inverse[i] = 2.0L*inverse[i] - buf2[i];
    }
}
