#include <algorithm>

#include "npstat/nm/findRootUsingBisections.hh"

#include "npstat/stat/AbsUnbinnedGOFTest1D.hh"

namespace npstat {
    void AbsUnbinnedGOFTest1D::simulateStatistic(
        AbsRandomGenerator& g, const unsigned long sz,
        const unsigned nPseudo, std::vector<double>* stats) const
    {
        if (!sz) throw std::invalid_argument(
            "In npstat::AbsUnbinnedGOFTest1D::simulateStatistic: "
            "sample size must be positive");
        if (nPseudo < 2U) std::invalid_argument(
            "In npstat::AbsUnbinnedGOFTest1D::simulateStatistic: "
            "insufficient number od pseudo experiments");
        assert(stats);
        stats->clear();
        stats->reserve(nPseudo);

        std::vector<double> sampleVec(sz);
        double* sample = &sampleVec[0];
        for (unsigned ips=0; ips<nPseudo; ++ips)
        {
            for (unsigned long i=0; i<sz; ++i)
                distro_->random(g, sample+i);
            std::sort(sampleVec.begin(), sampleVec.end());
            stats->push_back(this->testStatistic(sample, sz, true));
        }
        std::sort(stats->begin(), stats->end());
    }

    double AbsUnbinnedGOFTest1D::inverseExceedance(
        const double pvalue, const unsigned long sz,
        const double smin, const double smax) const
    {
        static const double tol = 1.0e-14;

        if (!hasAnalyticPValue()) throw std::runtime_error(
            "In npstat::AbsUnbinnedGOFTest1D::inverseExceedance: "
            "analytic p-value calculation is not implemented by "
            "the derived class");

        if (pvalue <= 0.0 || pvalue >= 1.0) throw std::invalid_argument(
            "In npstat::AbsUnbinnedGOFTest1D::inverseExceedance: "
            "p-value argument must be inside (0, 1) interval");

        double q = 0.0;
        if (!findRootUsingBisections(
                GOFTest1DPVFunctor(*this, sz), pvalue,
                smin, smax, tol, &q)) throw std::invalid_argument(
                    "In npstat::AbsUnbinnedGOFTest1D::inverseExceedance: "
                    "the initial interval does not bracket the root");
        return q;
    }
}
