#ifndef NPSTAT_DISCRETEGAUSS1DBUILDER_HH_
#define NPSTAT_DISCRETEGAUSS1DBUILDER_HH_

/*!
// \file DiscreteGauss1DBuilder.hh
//
// \brief Builder of discrete Gaussian filters in one dimension
//
// The filters are intended for use with the "LocalPolyFilter1D" class.
//
// Author: I. Volobouev
//
// December 2019
*/

#include <vector>

#include "npstat/stat/AbsFilter1DBuilder.hh"
#include "npstat/nm/ConvolutionEngine1D.hh"

namespace npstat {
    class DiscreteGauss1DBuilder : public AbsFilter1DBuilder
    {
    public:
        /**
        // The constructor arguments are as follows:
        //
        // bandwidth         -- The bandwidth of the filter in the units
        //                      of bin width. Must be non-negative.
        //
        // dataLen           -- Length of data arrays to be filtered by
        //                      the filters built with this object.
        */
        DiscreteGauss1DBuilder(double bandwidth, unsigned dataLen);

        inline virtual ~DiscreteGauss1DBuilder() {}

        /** A simple inspector of object properties */
        inline double bandwidth() const {return bandwidth_;}

        /** This method is pure virtual in the base, so it must be implemented */
        inline unsigned centralWeightLength() const {return dataLen_;}

        /** Internal filters are all different */
        inline bool keepAllFilters() const {return true;}

        /** The "taper" and "lenTaper" arguments will be ignored */
        virtual PolyFilter1D* makeFilter(const double* taper, unsigned lenTaper,
                                       unsigned binnum, unsigned datalen) const;
    private:
        DiscreteGauss1DBuilder();
        DiscreteGauss1DBuilder(const DiscreteGauss1DBuilder&);
        DiscreteGauss1DBuilder& operator=(const DiscreteGauss1DBuilder&);

        ConvolutionEngine1D engine_;
        std::vector<double> bufvec_;
        double bandwidth_;
        unsigned dataLen_;
    };
}

#endif // NPSTAT_DISCRETEGAUSS1DBUILDER_HH_
