#include <climits>

#include "npstat/stat/StatUtils.hh"

static double ADf(const double z, const int j)
{
    const double t = (4*j+1)*(4*j+1)*1.23370055013617/z;
    if (t > 150.0)
        return 0.0;
    double a = 2.22144146907918*exp(-t)/sqrt(t);
    // double b = 3.93740248643060*2.*cPhi(sqrt(2*t)); /* requires cPhi */
    /*if you have erfc(), replace 2*cPhi(sqrt(2*t)) with erfc(sqrt(t))*/
    double b = 3.93740248643060*erfc(sqrt(t));
    double r = z*.125;
    double f = a+b*r;
    for (int i=1; i<200; i++)
    {
        const double c = ((i-.5-t)*b+t*a)/i;
        a=b; b=c;
        r *= z/(8*i+8);
        if (fabs(r) < 1.e-40 || fabs(c) < 1.e-40)
            return f;
        const double fnew = f+c*r;
        if (f == fnew)
            return f;
        f = fnew;
    }
    return f;
}

static double ADinf(const double z)
{
    if (z < 0.01)
        return 0.0; /* avoids exponent limits; ADinf(.01)=.528e-52 */
    if (z > 32.0)
        return 1.0;

    double r = 1.0/z;
    double ad = r*ADf(z, 0);
    for (int j=1; j<100; j++)
    {
        r *= (0.5 - j)/j;
        const double adnew = ad+(4*j+1)*r*ADf(z,j);
        if (ad == adnew)
            return ad;
        ad = adnew;
    }
    return ad;
}

namespace npstat {
    double aicc(const double k, const double logli, const double n)
    {
        const double aic = 2.0*k - 2.0*logli;
        const double denom = n - k - 1;
        if (denom > 0.0)
            return aic + 2.0*k*(k + 1.0)/denom;
        else if (k > 0.0)
            return DBL_MAX;
        else
            return aic;
    }

    double AD(const double n, const double z)
    {
        const double x = ADinf(z);

        /* now x=adinf(z). Next, get v=errfix(n,x) and return x+v; */
        if (x > 0.8)
        {
            const double v=(-130.2137+(745.2337-(1705.091-(1950.646-(1116.360-255.7844*x)*x)*x)*x)*x)/n;
            return x+v;
        }

        const double c=.01265+.1757/n;
        if (x < c)
        { 
            double v = x/c;
            v=sqrt(v)*(1.-v)*(49*v-102);
            return x+v*(.0037/n/n + .00078/n + .00006)/n;
        }
        else
        {
            double v = (x-c)/(.8-c);
            v=-.00022633+(6.54034-(14.6538-(14.458-(8.259-1.91864*v)*v)*v)*v)*v;
            return x+v*(.04213+.01365/n)/n;
        }
    }
}
